package fr.sanchezm.attestationsCovid19.ui.add

import android.annotation.SuppressLint
import android.util.Log
import androidx.lifecycle.LiveData
import androidx.lifecycle.MutableLiveData
import androidx.lifecycle.ViewModel
import fr.sanchezm.attestationsCovid19.R
import fr.sanchezm.attestationsCovid19.data.db.entity.Profile
import fr.sanchezm.attestationsCovid19.data.repository.AttestationRepository
import fr.sanchezm.attestationsCovid19.data.repository.ProfileRepository
import fr.sanchezm.attestationsCovid19.utilities.Event

class AddViewModel(
    private val profileRepository: ProfileRepository,
    private val attestationRepository: AttestationRepository
) : ViewModel() {

    private val _errorMessage = MutableLiveData<Event<Int>>()

    val errorMessage: LiveData<Event<Int>>
        get() = _errorMessage

    val firstName = MutableLiveData<String>()
    val lastName = MutableLiveData<String>()
    val birthday = MutableLiveData<String>()
    val birthPlace = MutableLiveData<String>()
    val address = MutableLiveData<String>()
    val city = MutableLiveData<String>()
    val postalCode = MutableLiveData<String>()
    val exitDate = MutableLiveData<String>()
    val exitHour = MutableLiveData<String>()

    val reason1 = MutableLiveData(false)
    val reason2 = MutableLiveData(false)
    val reason3 = MutableLiveData(false)
    val reason4 = MutableLiveData(false)
    val reason5 = MutableLiveData(false)
    val reason6 = MutableLiveData(false)
    val reason7 = MutableLiveData(false)

    @SuppressLint("LongLogTag")
    fun onGenerateAttestationClick() {
        if (checkAllValue()) {
            profileRepository.insertProfile(getProfileFromView())
            attestationRepository.createAttestation()
        } else {
            _errorMessage.value = Event(R.string.error_cannot_create_attestation)
            Log.e("onGenerateAttestationClick", "Cannot generate Attestation")
        }
    }

    init {
        setProfileValue()
    }

    private fun setProfileValue() {
        val profile = profileRepository.getProfile()

        firstName.value = profile.value?.firstName
        lastName.value = profile.value?.lastName
        birthday.value = profile.value?.birthday
        birthPlace.value = profile.value?.birthPlace
        address.value = profile.value?.address
        city.value = profile.value?.city
        postalCode.value = profile.value?.postalCode
    }

    private fun getProfileFromView(): Profile {
        return Profile(
            firstName.value.toString(),
            lastName.value.toString(),
            birthday.value.toString(),
            birthPlace.value.toString(),
            address.value.toString(),
            city.value.toString(),
            postalCode.value.toString()
        )
    }

    private fun checkAllValue(): Boolean {
        return !firstName.value.isNullOrEmpty()
                && !lastName.value.isNullOrEmpty()
                && !birthday.value.isNullOrEmpty()
                && !birthPlace.value.isNullOrEmpty()
                && !address.value.isNullOrEmpty()
                && !city.value.isNullOrEmpty()
                && !postalCode.value.isNullOrEmpty()
                && !exitDate.value.isNullOrEmpty()
                && !exitHour.value.isNullOrEmpty()
                && checkAtLeastOneReasonSelected()
    }

    private fun checkAtLeastOneReasonSelected(): Boolean {
        return reason1.value!!
                || reason2.value!!
                || reason3.value!!
                || reason4.value!!
                || reason5.value!!
                || reason6.value!!
                || reason7.value!!
    }

}